# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IHasDeviceStates",
    "IDeviceState",
    "IDeviceStateSet",
    "IDeviceStateTarget",
    "DeviceStateSearchParams",
    ]

from zope.interface import Interface
from zope.schema import Datetime

from lazr.restful.declarations import (
    export_as_webservice_entry,
    exported,
    export_read_operation,
    operation_parameters,
    operation_returns_collection_of,
    )
from lazr.restful.frameworks.django import IDjangoLocation
from lazr.restful.interface import copy_field

from lpresults.registry.interfaces.search import (
    ISearchSet,
    SearchParams,
    )

from lpresults.hardware.interfaces.device import IDevice
from lpresults.hardware.interfaces.devicedriver import IHasDeviceDriver


class IDeviceState(IDjangoLocation, IDevice, IHasDeviceDriver):
    """A device state for a particular system unit."""

    export_as_webservice_entry()

    date_created = exported(
        Datetime(
            title=u"Date created",
            required=True,
            readonly=True,
            description=u"Date on which this device state was created."))


class IDeviceStateSet(ISearchSet):

    pass


class IHasDeviceStates(Interface):
    """An entity which has a collection of device states."""

    @operation_parameters(
        date_created=Datetime(
            title=u"Search devices created on the given date",
            required=False),
        created_before=Datetime(
            title=u"Search devices created before or on the given date",
            required=False),
        created_after=Datetime(
            title=u"Search devices created after the given date",
            required=False),
        )
    @operation_returns_collection_of(IDeviceState)
    @export_read_operation()
    def getDeviceStates(
        date_created=None, created_before=None, created_after=None):
        """Get the device states for this target."""


class IDeviceStateTarget(IHasDeviceStates):
    """An entity on which a device states can be added.

    Examples include an ISystemUnit for now.
    """
    export_as_webservice_entry()

    # XXX: Use read operation instead of write or factory to prevent
    #      lazr.restfulclient from making two calls per operation.
    @operation_parameters(
        bus_name=copy_field(IDeviceState["bus_name"]),
        category_name=copy_field(IDeviceState["category_name"]),
        product_name=copy_field(IDeviceState["product_name"]),
        vendor_name=copy_field(IDeviceState["vendor_name"]),
        product_id=copy_field(IDeviceState["product_id"]),
        vendor_id=copy_field(IDeviceState["vendor_id"]),
        subproduct_id=copy_field(IDeviceState["subproduct_id"]),
        subvendor_id=copy_field(IDeviceState["subvendor_id"]),
        driver_name=copy_field(IDeviceState["driver_name"]),
        date_created=copy_field(IDeviceState["date_created"]),
        )
    @export_read_operation()
    def addDeviceState(
        bus_name, category_name=None, product_name=None, vendor_name=None,
        product_id=None, vendor_id=None, subproduct_id=None,
        subvendor_id=None, driver_name=None, date_created=None):
        """Add a new device state on this target."""


class DeviceStateSearchParams(SearchParams):
    """Encapsulates search parameters for IDeviceStateSet.search()"""

    def __init__(
        self, date_created=None, created_before=None, created_after=None,
        **kwargs):
        super(DeviceStateSearchParams, self).__init__(**kwargs)
        self.date_created = date_created
        self.created_before = created_before
        self.created_after = created_after
