# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "System",
    "system_set",
    ]

from storm.locals import (
    AutoReload,
    Int,
    RawStr,
    Unicode,
    )

from zope.component import getUtility
from zope.event import notify
from zope.interface import implements

from lazr.lifecycle.event import ObjectCreatedEvent
from lazr.restful.interfaces import IServiceRootResource
from lazr.restful.simple import TraverseWithGet

from lpresults.database.search import search_to_expression
from lpresults.database.textgrams import get_text_grams_as_text
from lpresults.database.textsearch import (
    TSMatch,
    TSVectorPart,
    TSVectorValidator,
    get_grams_query,
    )

from lpresults.registry.models.search import SearchSet

from lpresults.hardware.interfaces.system import (
    ISystem,
    ISystemSet,
    SystemSearchParams,
    )
from lpresults.hardware.models.store import get_hardware_store
from lpresults.hardware.models.unit import UnitTargetMixin


SYSTEM_VALIDATOR = TSVectorValidator(
    "text_search_vector",
    TSVectorPart(
        u"model", weight=u"A", transform=(get_text_grams_as_text,)),
    TSVectorPart(
        u"make", weight=u"B", transform=(get_text_grams_as_text,)),
    TSVectorPart(
        u"version", weight=u"C", transform=(get_text_grams_as_text,)))


class System(UnitTargetMixin):

    __storm_table__ = "system"

    implements(ISystem)

    id = Int(primary=True, allow_none=False, default=AutoReload)
    model = Unicode(allow_none=False, validator=SYSTEM_VALIDATOR)
    make = Unicode(validator=SYSTEM_VALIDATOR)
    version = Unicode(validator=SYSTEM_VALIDATOR)

    text_search_vector = RawStr()

    def __init__(self, model, make=None, version=None):
        self.model = model
        self.make = make
        self.version = version

    @property
    def __parent__(self):
        return getUtility(IServiceRootResource)

    @property
    def __path_parts__(self):
        return ("systems", str(self.id),)

    @property
    def name(self):
        name = self.model
        if self.make:
            name = u"%s %s" % (self.make, name)
        if self.version:
            name = u"%s - %s" % (name, self.version)

        return name


class SystemSet(TraverseWithGet, SearchSet):

    implements(ISystemSet)

    search_params_factory = SystemSearchParams

    # See ITopLevelEntryLink
    link_name = "systems"
    entry_type = ISystemSet

    __url_path__ = "systems"

    @property
    def __parent__(self):
        return getUtility(IServiceRootResource)

    def retrieve(self, model, make=None, version=None):
        """See `ISystemSet`."""
        store = get_hardware_store()
        system = store.find(
            System, model=model, make=make, version=version).one()
        if not system:
            system = System(model, make, version)
            store.add(system)

        notify(ObjectCreatedEvent(system))

        return system

    def get(self, request, name):
        """See `ITraverseWithGet`."""
        id = int(name)
        return self.getById(id)

    def getById(self, id):
        """See `ISystemSet`."""
        return get_hardware_store().get(System, id)

    def search(self, *args):
        if not args:
            search_params = self.search_params_factory()
            args = (search_params,)

        return super(SystemSet, self).search(*args)

    def buildSearchExpressions(self, search_params):
        """See SearchSet."""
        expressions = super(SystemSet, self).buildSearchExpressions(
            search_params)

        if search_params.make is not None:
            expressions.append(
                search_to_expression(System.make, search_params.make))

        if search_params.model is not None:
            expressions.append(
                search_to_expression(System.model, search_params.model))

        return expressions

    def buildSearchText(self, search_text):
        """See SearchSet."""
        expressions = super(SystemSet, self).buildSearchText(search_text)

        text_search_query = get_grams_query(search_text)
        expressions.append(
            TSMatch(text_search_query, System.text_search_vector))

        return expressions

    def getSearchResultTables(self):
        """See SearchSet."""
        return System

    def getSearchStore(self, search_params):
        """See SearchSet."""
        return get_hardware_store()


system_set = SystemSet()
