# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "LaunchpadMixin",
    ]

import shutil

from os import path
from optparse import OptionGroup
from urlparse import urlsplit

# Make launchpadlib happy
try:
    import lazr.restful
except ImportError:
    pass

from lazr.restfulclient import _browser
from lazr.restfulclient.errors import HTTPError

from launchpadlib.launchpad import Launchpad

from lpresults.scripts.application import ApplicationError
from lpresults.scripts.functor import Functor
from lpresults.scripts.retry import retry_except


class LaunchpadMixin:

    # Launchpad defaults
    default_application_name = "Launchpad Results"
    default_launchpadlib_dir = "~/.lpresults"
    default_service_root = "http://localhost:8000/+api/"
    default_version = "1.0"

    def _getCache(self, index=None):
        """Return the path to the cache based on the Launchpad factory.

        :param index: Optionally, the index of the child process.
        """
        kwargs = self.launchpad_factory.kwargs
        if "cache" in kwargs:
            service_root_dir = path.dirname(kwargs["cache"])
        else:
            launchpadlib_dir = path.expanduser(kwargs["launchpadlib_dir"])
            host_name = urlsplit(kwargs["service_root"])[1]
            service_root_dir = path.join(launchpadlib_dir, host_name)

        if index is not None:
            service_root_dir = path.join(service_root_dir, str(index))

        return path.join(service_root_dir, "cache")

    def getLaunchpad(self, *args, **kwargs):
        """Helper to get a launchpad instance based on application options."""
        # Cache is broken with multiple processes, so each child process
        # needs to be given a different cache directory but the same
        # credentials can be shared safely.
        #
        # See: https://bugs.launchpad.net/launchpadlib/+bug/459418
        index = kwargs.pop("index", None)
        if index is not None:
            child_cache = self._getCache(index)
            kwargs.setdefault("cache", child_cache)
            if not path.exists(child_cache):
                parent_cache = self._getCache()
                shutil.copytree(parent_cache, child_cache)

        try:
            return self.launchpad_factory(*args, **kwargs)
        except AttributeError:
            # In stock httplib2, this exception is raised when failing
            # to connect to the remote host:
            #
            #   AttributeError: 'NoneType' object has no attribute 'makefile'
            #
            # See: http://code.google.com/p/httplib2/issues/detail?id=62
            raise ApplicationError(
                "Failed network connection to service root")

        except HTTPError, error:
            # When launchpadlib fails connecting to the service root,
            # this is an example of exception raised:
            #
            #   HTTPError: HTTP Error 504: Gateway Time-out
            raise ApplicationError(
                "Failed HTTP connection to service root: %s"
                % error.response.reason)

    def addOptions(self, parser):
        """See Application."""
        super(LaunchpadMixin, self).addOptions(parser)

        group = OptionGroup(parser, "Launchpad options")
        group.add_option("--application-name",
            default=self.default_application_name,
            metavar="NAME",
            help="The name of this application,"
                " defaults to: %default")
        group.add_option("--credentials-file",
            metavar="FILE",
            help="If given, the credentials are stored in that file.")
        group.add_option("--launchpadlib-dir",
            default=self.default_launchpadlib_dir,
            metavar="DIR",
            help="The directory for cache and credentials,"
                " defaults to: %default")
        group.add_option("--service-root",
            default=self.default_service_root,
            metavar="URL",
            help="The URL to the root of the web service,"
                " defaults to: %default")
        group.add_option("--version",
            default=self.default_version,
            help="The version of the web service,"
                " defaults to: %default")
        parser.add_option_group(group)

    def parseOptions(self, options, args):
        """See Application."""
        super(LaunchpadMixin, self).parseOptions(options, args)

        self.launchpad_factory = Functor(
            Launchpad.login_with, options.application_name,
            service_root=options.service_root,
            launchpadlib_dir=options.launchpadlib_dir,
            credentials_file=options.credentials_file,
            version=options.version)

    def preProcess(self):
        """See Application."""
        super(LaunchpadMixin, self).preProcess()

        # Establish an early Launchpad connection to share credentials
        # across multiple processes.
        launchpad = self.getLaunchpad()
        cache = self._getCache()
        kwargs = dict(
            service_root=self.launchpad_factory.kwargs["service_root"],
            version=self.launchpad_factory.kwargs["version"],
            credentials=launchpad.credentials,
            cache=cache)
        if hasattr(launchpad, "credential_store"):
            kwargs.update(
                authorization_engine=launchpad.authorization_engine,
                credential_store=launchpad.credential_store)

        self.launchpad_factory = Functor(Launchpad, **kwargs)


# An application can sometimes crash with a 5xx error in
# Launchpad.Browser._request(). This monkey patch retries
# a few times before giving up.
_original_request = _browser.Browser._request


def retry_request(*args, **kwargs):
    try:
        return _original_request(*args, **kwargs)
    except HTTPError, error:
        # 404 errors indicate a data inconsistency: more than one
        # try to open the URL is pointless.
        if error.response.status == 404:
            raise

        raise ApplicationError(str(error))

_browser.Browser._request = retry_except(ApplicationError)(
    retry_request)
