/* Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 *
 * The Form module provides classes for rendering forms.
 *
 * @module lpresults.ui
 * @submodule form
 */
YUI.add('lpresults.ui.form', function(Y) {

    var namespace = Y.namespace('lpresults.ui');

    Y.FormField.REQUIRED_ERROR_TEXT = 'Required input is missing.';

    var BUTTON_ATTRS = {
            primary: {
                value: false,
                validator: Y.Lang.isBoolean
            }
        },
        BUTTON_PROTOTYPE = {
            BOUNDING_TEMPLATE: '<span></span>',
            IS_ACTION: true,

            _syncPrimary: function() {
                if (this.get('primary')) {
                    this._fieldNode.addClass('primary');
                }
            }
        },
        FIELD_ATTRS = {
            focus: {
                value: false,
                validator: Y.Lang.isBoolean
            },
            requiredLabel: {
                value: '(Required)',
                validator: Y.Lang.isString
            }
        },
        FIELD_PROTOTYPE = {
            CONTENT_TEMPLATE: ['<div>',
                               '  <span class="label"></span>',
                               '  <div class="form-field">',
                               '    <div class="form-widget">',
                               '      <span class="field"></span>',
                               '    </div>',
                               '  </div>',
                               '  <span class="formHelp"></span>',
                               '</div>'].join(''),
            ERROR_CLASS : 'message',
            HINT_CLASS: 'formHelp',
            HINT_TEMPLATE: "<div></div>",

            _syncFocus: function() {
                if (this.get('focus')) {
                    this._fieldNode.focus();
                }
            },

            _showError: function() {
                // We have to render the custom markup for the error node here
                // because otherwise our CSS rules will catch it and render the
                // error icon.
                var contentBox = this.get('contentBox'),
                    formField = contentBox.one('.form-field');
                formField.append('<div class="error">' +
                                 '  <span class="message"></span>' +
                                 '</div>');
                Y.FormField.prototype._showError.apply(this, arguments);
            },

            _clearError: function() {
                Y.FormField.prototype._clearError.apply(this, arguments);
                var contentBox = this.get('contentBox'),
                    formError = contentBox.one('.error');
                if (formError) {
                    formError.remove();
                }
            }
        };

    /**
     * An hidden field node.
     *
     * @class HiddenField
     * @constructor
     */
    var HiddenField = function() {
        HiddenField.superclass.constructor.apply(this, arguments);
    };

    HiddenField.NAME = 'hidden-field';

    Y.extend(HiddenField, Y.HiddenField);

    namespace.HiddenField = HiddenField;

    /**
     * A text field node.
     *
     * @class TextField
     * @constructor
     */
    var TextField = function() {
        TextField.superclass.constructor.apply(this, arguments);
        this.after(this._syncFocus, this, 'syncUI');
    };

    TextField.NAME = 'text-field';
    TextField.ATTRS = Y.merge(FIELD_ATTRS, {
        /**
         * Value of 'size' attribute of the text field element.
         *
         * @attribute size
         * @type Number
         */
        size : {
            validator : Y.Lang.isNumber,
            value : null
        }
    });

    Y.extend(TextField, Y.TextField, Y.merge(FIELD_PROTOTYPE, {
        _syncFieldNode: function() {
            Y.TextField.prototype._syncFieldNode.apply(this, arguments);
            var size = this.get('size');
            if (size) {
                this._fieldNode.setAttribute('size', size);
            }
        }
    }));

    namespace.TextField = TextField;

    /**
     * A textarea field node.
     *
     * @class TextareaField
     * @constructor
     */
    var TextareaField = function() {
        TextareaField.superclass.constructor.apply(this, arguments);
    };

    TextareaField.NAME = 'textarea-field';
    TextareaField.ATTRS = Y.merge(FIELD_ATTRS);

    Y.extend(TextareaField, Y.TextareaField, Y.merge(FIELD_PROTOTYPE, {
        FIELD_TEMPLATE: '<textarea rows="15" cols="60"></textarea>'
    }));

    namespace.TextareaField = TextareaField;

    /**
     * A submit button widget. When pressed, the default action is to submit
     * the underlying form.
     *
     * @class SubmitButton
     * @constructor
     */
    var SubmitButton = function() {
        SubmitButton.superclass.constructor.apply(this, arguments);
        this.after(this._syncPrimary, this, 'syncUI');
    };

    SubmitButton.NAME = 'submit-button';
    SubmitButton.ATTRS = Y.merge(BUTTON_ATTRS);

    Y.extend(SubmitButton, Y.SubmitButton, Y.merge(BUTTON_PROTOTYPE));

    namespace.SubmitButton = SubmitButton;

    /**
     * A form button widget. When pressed, the default action is to execute
     * the callback function set with the 'onclick' attribute.
     *
     * @class FormButton
     * @constructor
     */
    var FormButton = function(config) {
        if (!config.value) {
            // As we use an 'input' element instead of a 'button' we need to
            // set the value attribute too, to make resetFieldNode behave
            // correctly
            config.value = config.label;
        }
        FormButton.superclass.constructor.apply(this, arguments);
        this.after(this._syncPrimary, this, 'syncUI');
    };

    FormButton.NAME = 'form-button';
    FormButton.ATTRS = Y.merge(BUTTON_ATTRS);

    Y.extend(FormButton, Y.FormButton, Y.merge(BUTTON_PROTOTYPE, {

        // Use an a input element instead of a plain button element.
        CONTENT_TEMPLATE: null,
        FIELD_TEMPLATE: '<input></input>',
        FIELD_CLASS: 'button',

        _renderFieldNode : function () {
            var contentBox = this.get('contentBox'),
                fieldNode = contentBox.one('input[type="submit"]');
            if (!fieldNode) {
                Y.FormButton.superclass._renderFieldNode.apply(this, arguments);
            } else {
                this._fieldNode = fieldNode;
            }
        },

        _syncFieldNode : function () {
            var label = this.get('label'),
                type = 'submit',
                name = label.replace(/ /g, '_').toLowerCase(),
                id = this.get('id') + Y.FormField.FIELD_ID_SUFFIX;

            this._fieldNode.setAttrs({
                name: name,
                type: type,
                id: id,
                value: label});
        }
    }));

    namespace.FormButton = FormButton;

    /**
     * A specialized Y.Form that uses a Launchpad Results markup structure
     *
     * @class Form
     * @constructor
     */
    var Form = function(config) {
        Form.superclass.constructor.apply(this, arguments);
        Y.after(this._renderActions, this, 'renderUI');
        Y.after(this._renderSummary, this, 'renderUI');
        Y.after(this._bindCheckAction, this, 'bindUI');
    };

    Form.NAME = 'form';
    Form.ATTRS = {
        /*
         * Optional summary header for the form.
         *
         * @attribute summary
         * @type String
         */
        summary: {
            validator: Y.Lang.isString,
            value: null
        },

        /*
         * Set to 'true' to divide fields and actions with a dotted line.
         */
        divide: {
            value: true
        }
    };

    Y.extend(Form, Y.Form, {

        ACTION_TEMPLATE: '<div></div>',
        SUMMARY_TEMPLATE: '<h2 class="form-summary"></h2>',

        _actionsNode: null,

        _bindCheckAction: function(evt) {
            this.after('addChild', this._checkAction);
        },

        _renderActions: function() {
            var controls;
            this.each(Y.bind(function (field) {
                if (field.IS_ACTION) {
                    this._renderAction(field);
                }
            }, this));
            this.each(Y.bind(function (field) {
                if (field.name == 'table-field') {
                    controls = [];
                    this._actionsNode.all('input').each(function(node) {
                        controls.push(node);
                    });
                }
            }, this));
        },

        _renderAction: function(field) {
            if (!this._actionsNode) {
                this._actionsNode = Y.Node.create(this.ACTION_TEMPLATE);
                if (this.get('divide')) {
                    this._actionsNode.addClass('form-actions');
                }
                this.get('contentBox').append(this._actionsNode);
            }
            this._actionsNode.append(field.get('boundingBox'));
        },

        _renderSummary: function(field) {
            var summary = this.get('summary'),
                header;
            if (summary) {
                header = Y.Node.create(this.SUMMARY_TEMPLATE);
                header.set('text', summary);
                this.get('contentBox').insert(header, 0);
            }
        },

        _checkAction: function(evt) {
            var field = evt.child;
            if (field.IS_ACTION) {
                 this._renderAction(field);
            }
        }
    });

    namespace.Form = Form;

}, '0.1', {'requires': ['gallery-form']});
