/* Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 *
 * The Searching module provides a class for searching results.
 *
 * @module lpresults.ui
 * @submodule searching
 */
YUI.add('lpresults.ui.searching', function(Y) {

    var namespace = Y.namespace('lpresults.ui');

    var Action = Y.lpresults.command.Action,
        ActionPlugin = Y.lpresults.command.ActionPlugin,
        Form = Y.lpresults.ui.Form,
        FormButton = Y.lpresults.ui.FormButton,
        TextField = Y.lpresults.ui.TextField,
        HOST = 'host',
        SEARCH_BOX = 'searchBox',
        SEARCH_TEXT = 'searchText';

    /**
     * Plugin renders searching and location status elements when the 'render'
     * event on the host widget fires.
     *
     * @class SearchingPlugin
     * @extends Plugin
     * @constructor
     */
    var SearchingPlugin = function() {
        SearchingPlugin.superclass.constructor.apply(this, arguments);
    };

    SearchingPlugin.NAME = 'searching_plugin';
    SearchingPlugin.NS = 'searching';
    SearchingPlugin.ATTRS = {
        /**
         * The element within which to render the search form.
         *
         * @attribute searchBox
         * @type Node
         */
        searchBox: {
            value: null
        },

        /**
         * The text to display as the search button, such as 'Submit'. Default
         * is 'Search'.
         *
         * @attribute searchText
         * @type String
         */
        searchText: {
            value: 'Search'
        },

        /**
         * The text to display as the search button after having pressed it,
         * such as 'Submit again'. Default is 'Search again'.
         *
         * @attribute searchAgainText
         * @type String
         */
        searchAgainText: {
            value: 'Search again'
        },

        /**
         * The hint to display underneath the search input field.
         *
         * @attribute searchHint
         * @type String
         */
        searchHint: {
            value: null
        },

        /**
         * The size of the search input field. Default is 30.
         *
         * @attribute searchSize
         * @type Number
         */
        searchSize: {
            value: 30
        }
    };

    Y.extend(SearchingPlugin, Y.Plugin.Base, {
        /**
         * Constructor logic.
         *
         * @method initializer
         * @protected
         */
        initializer: function(config) {
            if (config.searchText) {
                this.set(SEARCH_TEXT, config.searchText);
            }

            this._search_text = new TextField({
                hint: this.get('searchHint'),
                size: this.get('searchSize'),
                name: 'text',
                focus: true,
                required: true,
                requiredLabel: ''});
            this._search_button = new FormButton({
                label: this.get(SEARCH_TEXT)});
            this._form = new Form({
                divide: false,
                children: [
                    this._search_text,
                    this._search_button]});

            var host = this.get(HOST);
            host.on('before-content-change',
                Y.bind(this._renderUI, this));
            host.on('after-content-change',
                Y.bind(this._bindUI, this));
        },

        _renderUI: function() {
            var searchBox = this.get(SEARCH_BOX);
            this._form.render(searchBox);
        },

        _bindUI: function() {
            var action = new Action({
                client: Y.bind(this._search, this)});
            this._search_button.plug(ActionPlugin, {
                action: action,
                validate: true});
        },

        _search: function() {
            var text = this._search_text.get('value');
            this.get(HOST).runAction({text: text});

            this._search_button.set(
                'label', this.get('searchAgainText'));
            this._search_button.syncUI();
        }
    });

    /*
     * Ensure that SearchingPlugin is available within the ui namespace.
     */
    namespace.SearchingPlugin = SearchingPlugin;

}, '0.1', {'requires': [
    'plugin', 'lpresults.command.action', 'lpresults.ui.form']});
